//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ENBSeries effect file
// Visit http://enbdev.com for updates
// Copyright (c) 2007-2013 Boris Vorontsov
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Editing by Matso (SVI Series), 2013
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//+++++++++++++++++++++++++++++
// Internal parameters, can be modified
//+++++++++++++++++++++++++++++
// Lens & corona parameters.
#define USE_DOUBLE_CORONA
#define ENABLE_CORONA_COLOR_MAPPING
//#define ENABLE_LENS_DISPERSION

#define fCoronaScaleBias	1.0	// Corona scale bias

float	fLensIntensity 
<
	string UIName="SL: Lens intensity";
	string UIWidget="Spinner";
	float UIMin=0.0;
	float UIMax=10.0;
> = {1.0};

float	fLensContrast
<
	string UIName="SL: Lens contrast";
	string UIWidget="Spinner";
	float UIMin=0.0;
	float UIMax=2.0;
> = {1.2};

#ifdef ENABLE_LENS_DISPERSION
float	fLensDipersion
<
	string UIName="SL: Lens dispersion scale";
	string UIWidget="Spinner";
	float UIMin=0.0;
	float UIMax=1.0;
> = {0.02};
#endif

float	fCoronaIntensity 
<
	string UIName="SL: Corona intensity";
	string UIWidget="Spinner";
	float UIMin=0.0;
	float UIMax=10.0;
> = {1.0};

float	fCoronaContrast
<
	string UIName="SL: Corona contrast";
	string UIWidget="Spinner";
	float UIMin=0.0;
	float UIMax=2.0;
> = {0.8};

float	fCoronaScale
<
	string UIName="SL: Corona scale";
	string UIWidget="Spinner";
	float UIMin=-2.0;
	float UIMax=2.0;
> = {1.0};

#ifdef ENABLE_CORONA_COLOR_MAPPING
#define NUMBER_OF_COLORS	3//6		// Size of color map array.
#define ADDRESS_MODE		0//2		// Addressing mode for borders: 0 - clamp, 1 - mirror[not ready], 2 - wrap.

// Corona color mapping quasi-texture.
float4 colorMap[NUMBER_OF_COLORS] = {
// Define color map values here, thus many as NUMBER_OF_COLORS states (3 by default).
	{1.0, 0.0, 0.0, 1.0}
	//,{1.0, 1.0, 0.0, 1.0}
	,{0.0, 1.0, 0.0, 1.0}
	//,{0.0, 1.0, 1.0, 1.0}
	,{0.0, 0.0, 1.0, 1.0}
	//,{1.0, 0.0, 1.0, 1.0}
};

float	fCoronaColorMapFreq
<
	string UIName="CM: Corona color mapping frequency scale";
	string UIWidget="Spinner";
	float UIMin=1.0;
	float UIMax=20.0;
> = {2.5};//{2.0};

// Quasi-texture sampler function
// The quasi-texture is a simple array of RGBA color fragments, thus needs special treatment to be capable of texture-like usage.
float4 sample1DArrayNorm(float coord)	// Coordinate used here is a typical texture one, from range [0.0, 1.0]
{
	float4 color = 0.0;

	if (coord < 0.0 || coord > 1.0) {
// Clamp address mode.
#if (ADDRESS_MODE == 0)
		if (coord < 0.0) coord = 0.0;
		else coord = 1.0;
#endif

// TODO: Mirror address mode.
//#if (ADDRESS_MODE == 1)
//		int tm = -(int) ceil(coord);
//		int tp = (int) floor(coord);
//		if (coord < 0.0) {
//			if (coord > -1.0) coord = -coord;
//			else {
//				if (!(tm % 2)) coord = 
//				else // TODO
//			}
//		} else {
//			coord = 1.0;
//		}
//#endif

// Wrap address mode.
#if (ADDRESS_MODE == 2)
		if (coord > 1.0) coord = frac(coord);
		else coord = 1.0 - frac(abs(coord));
#endif
	}

	int scale = NUMBER_OF_COLORS - 1;
	int indexA = (int) floor(scale * coord);
	int indexB = (int) ceil(scale * coord);
	float factor = frac(scale * coord);
	
	float4 a = colorMap[indexA];
	float4 b = colorMap[indexB];
	
	color = lerp(a, b, factor);// TODO: Interpolation modes?
	
	return color;
}
#endif

//+++++++++++++++++++++++++++++
// External parameters, do not modify
//+++++++++++++++++++++++++++++
//keyboard controlled temporary variables (in some versions exists in the config file). Press and hold key 1,2,3...8 together with PageUp or PageDown to modify. By default all set to 1.0
float4	tempF1; //0,1,2,3
float4	tempF2; //5,6,7,8
float4	tempF3; //9,0
//x=Width, y=1/Width, z=ScreenScaleY, w=1/ScreenScaleY
float4	ScreenSize;
//x=generic timer in range 0..1, period of 16777216 ms (4.6 hours), w=frame time elapsed (in seconds)
float4	Timer;
//changes in range 0..1, 0 means full quality, 1 lowest dynamic quality (0.33, 0.66 are limits for quality levels)
float	EAdaptiveQualityFactor;
//xy=sun position on screen, w=visibility
float4	LightParameters;
//fov in degrees
float	FieldOfView;

//textures
texture2D texColor;//enbsunsprite.bmp/tga/png image file
texture2D texMask;//hdr color of sun masked by clouds or objects

sampler2D SamplerColor = sampler_state
{
	Texture   = <texColor>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerMask = sampler_state
{
	Texture   = <texMask>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

struct VS_OUTPUT_POST
{
	float4 vpos  : POSITION;
	float2 txcoord : TEXCOORD0;
};

struct VS_INPUT_POST
{
	float3 pos  : POSITION;
	float2 txcoord : TEXCOORD0;
};

//Several sprites moving similar to lenz effect,
//They are transformed in vertex shader and drawed separately for
//better performance. Offset is set in passes of technique
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
VS_OUTPUT_POST VS_Draw(VS_INPUT_POST IN, uniform float offset, uniform float scale)
{
	VS_OUTPUT_POST OUT;

	float4 pos = float4(IN.pos.x,IN.pos.y,IN.pos.z,1.0);
	pos.y *= ScreenSize.z;

	//create own parameters instead of this, including uv offsets
	float2 shift = LightParameters.xy * offset;
	pos.xy = pos.xy * scale - shift;

	OUT.vpos = pos;
	OUT.txcoord.xy = IN.txcoord.xy;

	return OUT;
}

float4 PS_Draw(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR
{
	float4 res = 0;
	float2 coord = IN.txcoord.xy;

	//read sun visibility as amount of effect
	float sunmask = tex2D(SamplerMask, float2(0.5, 0.5)).x;
	sunmask = pow(sunmask, fLensContrast);//more contrast to clouds
	clip(sunmask - 0.02);//early exit if too low
	
	sunmask *= LightParameters.w * fLensIntensity;
#ifdef ENABLE_LENS_DISPERSION
	float4 origcolor = tex2D(SamplerColor, coord.xy);
	float2 dispVec = fLensDipersion * normalize(IN.vpos.xy);
	origcolor.r = tex2D(SamplerColor, coord.xy + dispVec).r;
	origcolor.b = tex2D(SamplerColor, coord.xy - dispVec).b;
	res.rgb = origcolor * sunmask;	// I dunno why this line causes the effect to crash (or not even load)...
#else
	float4 origcolor = tex2D(SamplerColor, coord.xy);
	res.rgb = origcolor * sunmask;
#endif

	float clipper = dot(res.rgb, 0.333);
	clip(clipper - 0.0003);//skip draw if black

	res.a = 1.0;
	return res;
}

VS_OUTPUT_POST VS_Draw2(VS_INPUT_POST IN, uniform float scale)
{
	VS_OUTPUT_POST OUT;

	float4 pos = float4(IN.pos.x, IN.pos.y, IN.pos.z, 1.0);
	pos.y *= ScreenSize.z;
	
	float l = dot(LightParameters.xy, LightParameters.xy);
	float scaleFactor = (2.0 - 1.0 / (l + fCoronaScaleBias)) * fCoronaScale * scale;
	float2 shift = LightParameters.xy;
	pos.xy = pos.xy * scaleFactor + shift;

	OUT.vpos = pos;
	OUT.txcoord.xy = IN.txcoord.xy;

	return OUT;
}

float4 PS_Draw2(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR
{
	float4 res;
	float2 coord = IN.txcoord.xy;

	//read sun visibility as amount of effect
	float sunmask = tex2D(SamplerMask, float2(0.5, 0.5)).x;
	sunmask = pow(sunmask, fCoronaContrast);//more contrast to clouds
	clip(sunmask - 0.02);//early exit if too low
	
	float4 origcolor = tex2D(SamplerColor, coord.xy);	
	sunmask *= LightParameters.w * fCoronaIntensity;

#ifdef ENABLE_CORONA_COLOR_MAPPING
	float t = (1 - smoothstep(0.0, 0.5, distance(float2(0.5, 0.5), coord.xy))) * fCoronaColorMapFreq;
	float4 mappedColor = sample1DArrayNorm(t);
	res.xyz = origcolor.a * sunmask * mappedColor;
#else
	res.xyz = origcolor.a * sunmask;
#endif

	float clipper = dot(res.xyz, 0.333);
	clip(clipper - 0.0003);//skip draw if black

	res.w = 1.0;
	return res;
}

technique Draw
{
	pass P0
	{
		VertexShader = compile vs_3_0 VS_Draw(-0.5, 0.14);//offset, scale
		PixelShader  = compile ps_3_0 PS_Draw();

		AlphaBlendEnable=TRUE;
		SrcBlend=ONE;
		DestBlend=ONE;

		DitherEnable=FALSE;
		ZEnable=FALSE;
		CullMode=NONE;
		ALPHATESTENABLE=FALSE;
		SEPARATEALPHABLENDENABLE=FALSE;
		StencilEnable=FALSE;
		FogEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}

	pass P1
	{
		VertexShader = compile vs_3_0 VS_Draw(0.3, 0.3);//offset, scale
		PixelShader  = compile ps_3_0 PS_Draw();

		AlphaBlendEnable=TRUE;
		SrcBlend=ONE;
		DestBlend=ONE;

		DitherEnable=FALSE;
		ZEnable=FALSE;
		CullMode=NONE;
		ALPHATESTENABLE=FALSE;
		SEPARATEALPHABLENDENABLE=FALSE;
		StencilEnable=FALSE;
		FogEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}

	pass P2
	{
		VertexShader = compile vs_3_0 VS_Draw(1.8, 0.6);//offset, scale
		PixelShader  = compile ps_3_0 PS_Draw();

		AlphaBlendEnable=TRUE;
		SrcBlend=ONE;
		DestBlend=ONE;

		DitherEnable=FALSE;
		ZEnable=FALSE;
		CullMode=NONE;
		ALPHATESTENABLE=FALSE;
		SEPARATEALPHABLENDENABLE=FALSE;
		StencilEnable=FALSE;
		FogEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}
	
	pass P3
	{
		VertexShader = compile vs_3_0 VS_Draw2(1.0);
		PixelShader  = compile ps_3_0 PS_Draw2();

		AlphaBlendEnable=TRUE;
		SrcBlend=ONE;
		DestBlend=ONE;

		DitherEnable=FALSE;
		ZEnable=FALSE;
		CullMode=NONE;
		ALPHATESTENABLE=FALSE;
		SEPARATEALPHABLENDENABLE=FALSE;
		StencilEnable=FALSE;
		FogEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}
#ifdef USE_DOUBLE_CORONA	
	pass P4
	{
		VertexShader = compile vs_3_0 VS_Draw2(-0.45);
		PixelShader  = compile ps_3_0 PS_Draw2();

		AlphaBlendEnable=TRUE;
		SrcBlend=ONE;
		DestBlend=ONE;

		DitherEnable=FALSE;
		ZEnable=FALSE;
		CullMode=NONE;
		ALPHATESTENABLE=FALSE;
		SEPARATEALPHABLENDENABLE=FALSE;
		StencilEnable=FALSE;
		FogEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}
#endif
}